@@ -1,8 +1,7 @@
 /**************************************************************************
  *   cut.c  --  This file is part of GNU nano.                            *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mark Majeres                                      *
  *   Copyright (C) 2016 Benno Schulenberg                                 *
  *                                                                        *
@@ -49,10 +48,10 @@ inline bool keeping_cutbuffer(void)
 void cut_line(void)
 {
     if (openfile->current != openfile->filebot)
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current->next, 0);
     else
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current, strlen(openfile->current->data));
     openfile->placewewant = 0;
 }
@@ -60,15 +59,15 @@ void cut_line(void)
 #ifndef NANO_TINY
 /* Move all currently marked text into the cutbuffer, and set the
  * current place we want to where the text used to start. */
-void cut_marked(void)
+void cut_marked(bool *right_side_up)
 {
     filestruct *top, *bot;
     size_t top_x, bot_x;
 
     mark_order((const filestruct **)&top, &top_x,
-		(const filestruct **)&bot, &bot_x, NULL);
+		(const filestruct **)&bot, &bot_x, right_side_up);
 
-    move_to_filestruct(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
+    extract_buffer(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
     openfile->placewewant = xplustabs();
 }
 
@@ -82,20 +81,18 @@ void cut_to_eol(void)
 {
     size_t data_len = strlen(openfile->current->data);
 
-    assert(openfile->current_x <= data_len);
-
     if (openfile->current_x < data_len)
 	/* If we're not at the end of the line, move all the text from
 	 * the current position up to it, not counting the newline at
 	 * the end, into the cutbuffer. */
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current, data_len);
     else if (openfile->current != openfile->filebot) {
 	/* If we're at the end of the line, and it isn't the last line
 	 * of the file, move all the text from the current position up
 	 * to the beginning of the next line, i.e. the newline at the
 	 * end, into the cutbuffer. */
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current->next, 0);
 	openfile->placewewant = xplustabs();
     }
@@ -105,23 +102,17 @@ void cut_to_eol(void)
  * file into the cutbuffer. */
 void cut_to_eof(void)
 {
-    move_to_filestruct(&cutbuffer, &cutbottom,
+    extract_buffer(&cutbuffer, &cutbottom,
 		openfile->current, openfile->current_x,
 		openfile->filebot, strlen(openfile->filebot->data));
 }
 #endif /* !NANO_TINY */
 
-/* Move text from the current filestruct into the cutbuffer.  If
- * copy_text is TRUE, copy the text back into the filestruct afterward.
+/* Move text from the current buffer into the cutbuffer.  If
+ * copy_text is TRUE, copy the text back into the buffer afterward.
  * If cut_till_eof is TRUE, move all text from the current cursor
  * position to the end of the file into the cutbuffer. */
-void do_cut_text(
-#ifndef NANO_TINY
-	bool copy_text, bool cut_till_eof
-#else
-	void
-#endif
-	)
+void do_cut_text(bool copy_text, bool cut_till_eof)
 {
 #ifndef NANO_TINY
     filestruct *cb_save = NULL;
@@ -130,45 +121,39 @@ void do_cut_text(
 	/* The length of the string at the current end of the cutbuffer,
 	 * before we add text to it. */
     bool old_no_newlines = ISSET(NO_NEWLINES);
+    bool right_side_up = TRUE;
+	/* There *is* no region, *or* it is marked forward. */
 #endif
+    size_t was_totsize = openfile->totsize;
 
-    assert(openfile->current != NULL && openfile->current->data != NULL);
-
-    /* Empty the cutbuffer when a chain of cuts is broken. */
+    /* If a chain of cuts was broken, empty the cutbuffer. */
     if (!keep_cutbuffer) {
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
 #ifdef DEBUG
 	fprintf(stderr, "Blew away cutbuffer =)\n");
 #endif
+	/* Indicate that future cuts should add to the cutbuffer. */
+	keep_cutbuffer = TRUE;
     }
 
 #ifndef NANO_TINY
     if (copy_text) {
+	/* If the cutbuffer isn't empty, remember where it currently ends. */
 	if (cutbuffer != NULL) {
-	    /* If the cutbuffer isn't empty, save where it currently
-	     * ends.  This is where we'll add the new text. */
 	    cb_save = cutbottom;
 	    cb_save_len = strlen(cutbottom->data);
 	}
-
-	/* Set NO_NEWLINES to TRUE, so that we don't disturb the last
-	 * line of the file when moving text to the cutbuffer. */
+	/* Don't add a magicline when moving text to the cutbuffer. */
 	SET(NO_NEWLINES);
     }
-#endif
 
-    /* Ensure that the text we're going to move into the cutbuffer will
-     * be added to the text already there, instead of replacing it. */
-    keep_cutbuffer = TRUE;
-
-#ifndef NANO_TINY
     if (cut_till_eof) {
 	/* Move all text up to the end of the file into the cutbuffer. */
 	cut_to_eof();
     } else if (openfile->mark_set) {
 	/* Move the marked text to the cutbuffer, and turn the mark off. */
-	cut_marked();
+	cut_marked(&right_side_up);
 	openfile->mark_set = FALSE;
     } else if (ISSET(CUT_TO_END))
 	/* Move all text up to the end of the line into the cutbuffer. */
@@ -180,35 +165,35 @@ void do_cut_text(
 
 #ifndef NANO_TINY
     if (copy_text) {
-	/* Copy the text in the cutbuffer, starting at its saved end if
-	 * there is one, back into the filestruct.  This effectively
-	 * uncuts the text we just cut without marking the file as
-	 * modified. */
+	/* Copy the text that is in the cutbuffer (starting at its saved end,
+	 * if there is one) back into the current buffer.  This effectively
+	 * uncuts the text we just cut. */
 	if (cutbuffer != NULL) {
 	    if (cb_save != NULL) {
 		cb_save->data += cb_save_len;
-		copy_from_filestruct(cb_save);
+		copy_from_buffer(cb_save);
 		cb_save->data -= cb_save_len;
 	    } else
-		copy_from_filestruct(cutbuffer);
+		copy_from_buffer(cutbuffer);
 
-	    /* Set the current place we want to where the text from the
-	     * cutbuffer ends. */
-	    openfile->placewewant = xplustabs();
+	    /* If the copied region was marked forward, put the new desired
+	     * x position at its end; otherwise, leave it at its beginning. */
+	    if (right_side_up)
+		openfile->placewewant = xplustabs();
 	}
-
-	/* Set NO_NEWLINES back to what it was before, since we're done
-	 * disturbing the text. */
+	/* Restore the magicline behavior now that we're done fiddling. */
 	if (!old_no_newlines)
 	    UNSET(NO_NEWLINES);
     } else
 #endif /* !NANO_TINY */
+    /* Only set the modification flag if actually something was cut. */
+    if (openfile->totsize != was_totsize)
 	set_modified();
 
     refresh_needed = TRUE;
 
 #ifndef DISABLE_COLOR
-    reset_multis(openfile->current, FALSE);
+    check_the_multis(openfile->current);
 #endif
 
 #ifdef DEBUG
@@ -216,29 +201,30 @@ void do_cut_text(
 #endif
 }
 
-/* Move text from the current filestruct into the cutbuffer. */
+/* Move text from the current buffer into the cutbuffer. */
 void do_cut_text_void(void)
 {
 #ifndef NANO_TINY
     add_undo(CUT);
+#endif
     do_cut_text(FALSE, FALSE);
+#ifndef NANO_TINY
     update_undo(CUT);
-#else
-    do_cut_text();
 #endif
 }
 
 #ifndef NANO_TINY
-/* Move text from the current filestruct into the cutbuffer, and copy it
- * back into the filestruct afterward.  If the mark is set or the cursor
+/* Move text from the current buffer into the cutbuffer, and copy it
+ * back into the buffer afterward.  If the mark is set or the cursor
  * was moved, blow away previous contents of the cutbuffer. */
 void do_copy_text(void)
 {
     static struct filestruct *next_contiguous_line = NULL;
     bool mark_set = openfile->mark_set;
 
-    /* Remember the current view port and cursor position. */
+    /* Remember the current viewport and cursor position. */
     ssize_t is_edittop_lineno = openfile->edittop->lineno;
+    size_t is_firstcolumn = openfile->firstcolumn;
     ssize_t is_current_lineno = openfile->current->lineno;
     size_t is_current_x = openfile->current_x;
 
@@ -251,8 +237,9 @@ void do_copy_text(void)
     next_contiguous_line = (mark_set ? NULL : openfile->current);
 
     if (mark_set) {
-	/* Restore the view port and cursor position. */
+	/* Restore the viewport and cursor position. */
 	openfile->edittop = fsfromline(is_edittop_lineno);
+	openfile->firstcolumn = is_firstcolumn;
 	openfile->current = fsfromline(is_current_lineno);
 	openfile->current_x = is_current_x;
     }
@@ -267,43 +254,50 @@ void do_cut_till_eof(void)
 }
 #endif /* !NANO_TINY */
 
-/* Copy text from the cutbuffer into the current filestruct. */
+/* Copy text from the cutbuffer into the current buffer. */
 void do_uncut_text(void)
 {
-    int was_lineno = openfile->current->lineno;
+    ssize_t was_lineno = openfile->current->lineno;
+	/* The line number where we started the paste. */
+    size_t was_leftedge = 0;
+	/* The leftedge where we started the paste. */
 
-    assert(openfile->current != NULL && openfile->current->data != NULL);
-
-    /* If the cutbuffer is empty, get out. */
+    /* If the cutbuffer is empty, there is nothing to do. */
     if (cutbuffer == NULL)
 	return;
 
 #ifndef NANO_TINY
     add_undo(PASTE);
+
+    if (ISSET(SOFTWRAP))
+	was_leftedge = (xplustabs() / editwincols) * editwincols;
 #endif
 
-    /* Add a copy of the text in the cutbuffer to the current filestruct
+    /* Add a copy of the text in the cutbuffer to the current buffer
      * at the current cursor position. */
-    copy_from_filestruct(cutbuffer);
+    copy_from_buffer(cutbuffer);
 
 #ifndef NANO_TINY
     update_undo(PASTE);
 #endif
 
-    if (openfile->current->lineno - was_lineno < editwinrows)
+    /* If we pasted less than a screenful, don't center the cursor. */
+    if (less_than_a_screenful(was_lineno, was_leftedge))
 	focusing = FALSE;
 
-    /* Set the current place we want to where the text from the
-     * cutbuffer ends. */
+    /* Set the desired x position to where the pasted text ends. */
     openfile->placewewant = xplustabs();
 
     /* Mark the file as modified. */
     set_modified();
 
+    /* Update current_y to account for the inserted lines. */
+    place_the_cursor(TRUE);
+
     refresh_needed = TRUE;
 
 #ifndef DISABLE_COLOR
-    reset_multis(openfile->current, FALSE);
+    check_the_multis(openfile->current);
 #endif
 
 #ifdef DEBUG
