@@ -1,8 +1,7 @@
 /**************************************************************************
  *   search.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
  *   Copyright (C) 2015, 2016, 2017 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -144,7 +143,8 @@ int search_init(bool replacing, bool use_answer)
 
     /* This is now one simple call.  It just does a lot. */
     i = do_prompt(FALSE, FALSE,
-		replacing ? MREPLACE : MWHEREIS, backupstring,
+		inhelp ? MFINDINHELP : (replacing ? MREPLACE : MWHEREIS),
+		backupstring,
 #ifndef DISABLE_HISTORIES
 		&search_history,
 #endif
@@ -202,7 +202,7 @@ int search_init(bool replacing, bool use_answer)
 	TOGGLE(USE_REGEXP);
 	backupstring = mallocstrcpy(backupstring, answer);
 	return 1;
-    } else if (func == do_replace || func == flip_replace_void) {
+    } else if (func == flip_replace) {
 	backupstring = mallocstrcpy(backupstring, answer);
 	return -2;	/* Call the opposite search function. */
     } else if (func == do_gotolinecolumn_void) {
@@ -438,7 +438,6 @@ void go_looking(void)
 {
     filestruct *was_current = openfile->current;
     size_t was_current_x = openfile->current_x;
-    int didfind;
 #ifdef DEBUG
     clock_t start = clock();
 #endif
@@ -884,6 +883,12 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     openfile->current_x = actual_x(openfile->current->data, column - 1);
     openfile->placewewant = column - 1;
 
+#ifndef NANO_TINY
+    if (ISSET(SOFTWRAP) && openfile->placewewant / editwincols >
+			strlenpt(openfile->current->data) / editwincols)
+	openfile->placewewant = strlenpt(openfile->current->data);
+#endif
+
     /* When the position was manually given, center the target line. */
     if (interactive) {
 	adjust_viewport(CENTERING);
@@ -994,7 +999,7 @@ void do_find_bracket(void)
 	 * the bracket at the current cursor position. */
     int wanted_ch_len;
 	/* The length of wanted_ch in bytes. */
-    char *bracket_set;
+    char bracket_set[MAXCHARLEN * 2 + 1];
 	/* The pair of characters in ch and wanted_ch. */
     size_t i;
 	/* Generic loop variable. */
@@ -1008,8 +1013,6 @@ void do_find_bracket(void)
 	/* The initial bracket count. */
     bool reverse;
 	/* The direction we search. */
-    char *found_ch;
-	/* The character we find. */
 
     assert(mbstrlen(matchbrackets) % 2 == 0);
 
@@ -1057,20 +1060,16 @@ void do_find_bracket(void)
     wanted_ch_len = parse_mbchar(wanted_ch, NULL, NULL);
 
     /* Fill bracket_set in with the values of ch and wanted_ch. */
-    bracket_set = charalloc((mb_cur_max() * 2) + 1);
     strncpy(bracket_set, ch, ch_len);
     strncpy(bracket_set + ch_len, wanted_ch, wanted_ch_len);
     bracket_set[ch_len + wanted_ch_len] = '\0';
 
-    found_ch = charalloc(mb_cur_max() + 1);
-
     while (TRUE) {
 	if (find_bracket_match(reverse, bracket_set)) {
 	    /* If we found an identical bracket, increment count.  If we
 	     * found a complementary bracket, decrement it. */
-	    parse_mbchar(openfile->current->data + openfile->current_x,
-			found_ch, NULL);
-	    count += (strncmp(found_ch, ch, ch_len) == 0) ? 1 : -1;
+	    count += (strncmp(openfile->current->data + openfile->current_x,
+				 ch, ch_len) == 0) ? 1 : -1;
 
 	    /* If count is zero, we've found a matching bracket.  Update
 	     * the screen and get out. */
@@ -1088,10 +1087,6 @@ void do_find_bracket(void)
 	    break;
 	}
     }
-
-    /* Clean up. */
-    free(bracket_set);
-    free(found_ch);
 }
 #endif /* !NANO_TINY */
 
@@ -1235,7 +1230,7 @@ void get_history_older_void(void)
     ;
 }
 
-#ifndef DISABLE_TABCOMP
+#ifdef ENABLE_TABCOMP
 /* Move h to the next string that's a tab completion of the string s,
  * looking at only the first len characters of s, and return that
  * string.  If there isn't one, or if len is 0, don't move h and return
@@ -1288,5 +1283,5 @@ char *get_history_completion(filestruct **h, char *s, size_t len)
      * match, or len is 0.  Return s. */
     return (char *)s;
 }
-#endif /* !DISABLE_TABCOMP */
+#endif /* ENSABLE_TABCOMP */
 #endif /* !DISABLE_HISTORIES */
